#!/usr/bin/env python
#-*- coding:utf-8 -*-
#==============================================================================
#  Copyright (C) 2010-2012 Schaefer & Tobies SuC GmbH.
#  Author: Markus Grimm <mgr@schaefer-tobies.de>
#
#  This file is part of Pyontapi.
#
#  Pyontapi is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyontapi is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Pyontapi.  If not, see <http://www.gnu.org/licenses/>.
#
#==============================================================================

"""
This module is intended to be used for handling a bunch of
:class:`schtob.pyontapi.NaFiler` objects in an environment, where you have a
single user, which is allowed to run API commands on all your filers. So,
instead of manually creating multiple NaFiler objects, you can use the Filers
class. The only thing you have to do is defining the ``NA_CONFIG`` dict in
:mod:`schtob.pyontapi.settings`.

Once you have defined the ``NA_CONFIG`` dict, you can access a
:class:`schtob.pyontapi.NaFiler` object using the
:class:`schtob.pyontapi.Filers` class like this::

    >>> from schtob.pyontapi import Filers, errors
    >>> try:
    ...     result = Filers('filer-name').system.get_ontapi_version()
    ... except errors.APIFailure, exc:
    ...     print exc.get_error()
    ...
    >>> print result
    {'major-version': 1, 'minor-version': 9}

"""

import copy

from schtob.pyontapi.na_filer import NaFiler


class Filers(object):

    """
    Connections to multiple filers.

    Use it in a static way like this::

        Filers('my-filer').volume.list_info()
    """

    __filers = {}
    __config = {}
    __unset = True

    def __new__(cls, name, role='default'):
        # you can access a connection by typing Filers(filername).foo
        # Connection is created with default params, defined in NA_CONFIG
        return cls.get_connection(name, role)

    def setup_config(cls, config):
        """ .. versionadded:: 0.2.2

        Setup the configuration for all your Filers
        """
        cls.__config = config
        cls.__unset = False

    setup_config = classmethod(setup_config)

    def get_connection(cls, name, role='default'):
        """ Get connection to filer ``name``.

        .. versionadded:: 0.2.2
            The role parameter was added
        """
        if not (name, role) in cls.__filers:
            if cls.__unset:
                cls.__config =  __import__(
                    'schtob.pyontapi.settings', fromlist=['NA_CONFIG']
                ).NA_CONFIG
                cls.__unset = False
            roles = copy.deepcopy(cls.__config['roles'])
            filer_roles = cls.__config.get('filer-roles', {})

            rolesettings = roles['default']

            if name in filer_roles and 'default' in filer_roles[name]:
                rolesettings.update(filer_roles[name]['default'])

            if role in roles:
                rolesettings.update(roles[role])

            if name in filer_roles and role in filer_roles[name]:
                rolesettings.update(filer_roles[name][role])

            return cls.create_connection(name, rolesettings, role)
        return cls.__filers[(name, role)]

    get_connection = classmethod(get_connection)

    def create_connection(cls, name, settings, role='default'):
        """
        Create a connection to filer ``name`` with ``settings``
        Use this to instantiate a "special" connection where settings differ
        from ``NA_CONFIG`` instead of just using Filers(name).

        .. versionadded:: 0.2.2
            The role parameter was added
        """

        cls.__filers[(name, role)] = NaFiler(name, settings)
        return cls.__filers[(name, role)]

    create_connection = classmethod(create_connection)

    def drop_connection(cls, name, role='default'):
        """ Drops connection to filer ``name``. """
        if (name, role) in cls.__filers:
            cls.__filers.pop((name, role))

    drop_connection = classmethod(drop_connection)
