#!/usr/bin/env python
#-*- coding:utf-8 -*-
#==============================================================================
#  Copyright (C) 2010-2012 Schaefer & Tobies SuC GmbH.
#  Author: Markus Grimm <mgr@schaefer-tobies.de>
#
#  This file is part of Pyontapi.
#
#  Pyontapi is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyontapi is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Pyontapi.  If not, see <http://www.gnu.org/licenses/>.
#
#==============================================================================

""" Generator for API commands according to ONTAPI version. """

import sys

from schtob.pyontapi import errors, api, system


def generate(filer):
    """ generate API commands for ``filer``'s version. """

    package = system.System(filer)
    typedefs = gen_typedefs(package)
    return get_api_command_packages(package, typedefs)


def gen_typedefs(package):
    """ Generate typedef classes. """
    typedefs = {}
    try:
        result = package.get_api_list_types()
    except errors.APIFailure:
        exc = sys.exc_info()[1]
        raise errors.APIGenerationError(exc.errno,
            'cannot get api types! Error was:\n%s' % exc.reason
        )

    for element in result['type-entries']:
        typedefs[element['name']] = api.NamedType(
            element['name'], element['type-elements']
        )

    for typedef in typedefs.values():
        elements = []
        for element in typedef.elements:
            if element['type'] is None:
                element['type'] = 'string'
            var_type = element['type'].replace('[]', '')
            if var_type in api.GENERIC_TYPEDEFS:
                var_type = api.GENERIC_TYPEDEFS[var_type]
            else:
                var_type = typedefs[var_type]
            _element = api.TypeDef(element, var_type)
            elements.append(_element)
        typedef.elements = elements
    return typedefs


def get_api_command_packages(package, typedefs):
    """
    Get all api commands for ``filer``. Returns a dict containing all
    packages.
    """

    try:
        result = package.api_list()
    except errors.APIFailure:
        exc = sys.exc_info()[1]
        raise errors.OntapiVersionError(exc.errno,
                'Can not run system-api-list. Error was:\n%s' % exc.reason
        )

    try:
        result = package.api_get_elements([
            child['name'] for child in result['apis']
        ])
    except errors.APIFailure:
        exc = sys.exc_info()[1]
        raise errors.OntapiVersionError(exc.errno,
                    'Can not get api elements. Error was:\n%s' % exc.reason
        )

    packages = {}

    for child in result['api-entries']:
        name = child['name']
        elements = []
        if 'api-elements' in child:
            for element_info in child['api-elements']:
                info = {}
                for key, value in element_info.items():
                    info[key] = value
                var_type = info['type'].replace('[]', '')
                if var_type in api.GENERIC_TYPEDEFS:
                    var_type = api.GENERIC_TYPEDEFS[var_type]
                else:
                    var_type = typedefs[var_type].duplicate()
                typedef = api.TypeDef(info, var_type)
                if isinstance(var_type, api.NamedType):
                    var_type.set_output_value_for_childs(
                        typedef.is_optional
                    )
                elements.append(typedef)
        api_command = api.APICommand(name, elements)
        if not api_command.get_package() in packages:
            packages[api_command.get_package()] = []
        packages[api_command.get_package()].append(api_command)
    return packages
